import { Controller, Get } from '@nestjs/common';
import { AppService } from './app.service';
import { ApiOperation, ApiProperty, ApiResponse } from '@nestjs/swagger';
import { META } from './meta';
import { Auth, AuthData } from './auth/authdata.decorator';
import { AuthenticationData } from './auth/auth.guard';

export class ApiInfo {
  @ApiProperty()
  version: string;
}

export class AuthInfo {
  @ApiProperty()
  authenticated: boolean;

  @ApiProperty()
  user: string;

  @ApiProperty({ default: false })
  god: boolean;
}

@Controller()
export class AppController {
  constructor(private readonly appService: AppService) {}

  @Get()
  @ApiOperation({ summary: 'Fetch API configuration' })
  getHello(): ApiInfo {
    return {
      version: META.version,
    };
  }

  @Get('/auth')
  @Auth()
  @ApiOperation({ summary: 'Check authentication status' })
  @ApiResponse({ type: AuthInfo, status: 200 })
  getAuth(@AuthData() auth: AuthenticationData): AuthInfo {
    return {
      // Since the AuthGuard stops the request otherwise, we can assume the user is authenticated
      authenticated: true,
      user: auth.user,
      god: auth.god || false,
    };
  }
}
