import { Body, Controller, Ip, Post } from '@nestjs/common';
import { ApiProperty, ApiResponse, ApiTags } from '@nestjs/swagger';
import { NoAuthentication } from './authdata.decorator';
import { AuthService } from './auth.service';

export class LoginData {
  @ApiProperty({ description: 'The user ID you are trying to log in as' })
  user: string;
}

export class LoginResponse {
  @ApiProperty()
  token: string;
  @ApiProperty()
  otp: string;
}

@Controller('auth')
@ApiTags('authentication')
export class AuthController {
  constructor(private auth: AuthService) {}

  @Post('login')
  @NoAuthentication()
  @ApiResponse({ type: LoginResponse })
  async login(
    @Body() data: LoginData,
    @Ip() ip: string,
  ): Promise<LoginResponse> {
    return await this.auth.createLoginRequest(data.user, ip);
  }
}
