import { Injectable, InternalServerErrorException } from '@nestjs/common';
import { GetMutualServersRes } from 'lib';
import { DatabaseService } from 'src/database/database.service';
import { ServerDetails } from './servers.controller';

@Injectable()
export class ServersService {
  constructor(private db: DatabaseService) {}

  async fetchMutual(user: string): Promise<GetMutualServersRes> {
    try {
      return await this.db.getMessengers().getMutualServers({ user });
    } catch (e) {
      throw new InternalServerErrorException(e);
    }
  }

  async fetchServerDetails(
    server: string,
    user: string,
  ): Promise<ServerDetails> {
    const config = await this.db.getDb().servers.findOne({ _id: server });
    const data = await this.db
      .getMessengers()
      .getServerDetails({ server, user });

    return {
      info: {
        id: data.id,
        name: data.name,
        icon: data.icon,
        moderator: data.isModerator,
        discoverable: data.discoverable,
      },
      config: {
        prefix: config?.prefix,
        infraction_visibility: config?.infraction_visibility,
        language: config?.language,
        logs: {
          moderation: config?.logs?.moderation,
          messages: config?.logs?.messages,
        },
      },
    };
  }
}
