/* eslint-disable @typescript-eslint/no-non-null-assertion */
import { Logger } from '@nestjs/common';
import { startWatchingDataUpdate } from 'geoip-lite';
import { exec } from 'child_process';

export function maxmindUpdater() {
  startWatchingDataUpdate(() => {
    Logger.log(`GeoIP dataset has been updated`);
  });

  const apiKey = process.env.MAXMIND_API_KEY?.trim();
  if (apiKey?.match(/\s/g)) {
    // This should protect against command injection in case an attacker somehow controls the API key variable
    throw new Error(`Maxmind API key includes a whitespace`);
  }
  if (apiKey) {
    Logger.log(
      `Maxmind API key was provided; Background geoip updates are enabled`,
    );

    async function update() {
      Logger.log(`Checking for GeoIP updates`);
      const process = exec(
        `cd node_modules/geoip-lite && npm run-script updatedb license_key=${apiKey}`,
      );

      process.on('close', (code) => {
        if (code == 0) Logger.log('GeoIP update completed with no error');
        else Logger.warn('GeoIP update exited with exit code ' + code);
      });
    }

    if (process.env.NODE_ENV != 'development') {
      // GeoIP only updates files if the hash has changed, so it's fine to run this on every startup
      setTimeout(update, 1000 * 30);
      setInterval(update, 1000 * 60 * 60 * 24);
    } else {
      Logger.log(
        `Not scheduling GeoIP update tasks as we're running in development mode`,
      );
    }
  }
}
