import { Command, embedMsg } from "lib";
import { t } from "../../modules/i18n.js";

// This will at some point be rendered into HTML and displayed online.
// This command will eventually be entirely rewritten, but for now it works
export default {
    name: "help",
    aliases: ["commands"],
    description: "List all commands",
    async run(message, args, client, context) {
        // A top level import statement would cause a dependency loop here
        const { userHasCommandPermissions, commands } = await import(
            "../../modules/commands.js"
        );

        const result: string[] = [];

        for (const command of commands) {
            let line = `${context.serverPrefix}${command.name}`;
            if (command.serverOnly && !message.server) line = `~~${line}~~`;
            if (
                !(await userHasCommandPermissions(
                    command,
                    message.author!,
                    message.member,
                ))
            ) {
                line = `${line} 🔒`;
            }
            result.push(line);
        }

        return await message.reply(
            embedMsg(
                (await t("commands.help.header", message.author)) +
                    "\n" +
                    result.join("\n") +
                    "\n###### " +
                    (await t("commands.help.legend", message.author)),
                await t("commands.help.title", message.author),
                "INFO",
            ),
        );
    },
} as Command;
