import {
    Command,
    DBTimedAction,
    TimedActionState,
    TimedActionType,
    embedMsg,
    splitTimeInput,
} from "lib";
import { ulid } from "ulid";
import { scheduleEvent } from "../../modules/timedActions.js";
import { t } from "../../modules/i18n.js";

const MAX_TEXT_LENGTH = 1000;

export default {
    name: "remindme",
    aliases: ["reminder", "remind"],
    description: "Schedule a reminder message",
    async run(message, args, client, context) {
        const { time, text } = splitTimeInput(args);

        if (time < 1000) {
            return await message.reply(
                embedMsg(
                    await t("commands.remindme.no_time", message.author, {
                        prefix: context.serverPrefix,
                    }),
                    await t("commands.remindme.title", message.author),
                    "ERROR",
                ),
            );
        }
        if (text.length > MAX_TEXT_LENGTH) {
            return await message.reply(
                embedMsg(
                    await t(
                        "commands.remindme.message_too_long",
                        message.author,
                        {
                            length: MAX_TEXT_LENGTH,
                        },
                    ),
                    await t("commands.remindme.title", message.author),
                    "ERROR",
                ),
            );
        }

        const action: DBTimedAction = {
            _id: ulid(),
            action: TimedActionType.Reminder,
            date: new Date(Date.now() + time),
            reminder: text,
            state: TimedActionState.Pending,
            user: message.authorId!,
            message: message.url,
        };

        await scheduleEvent(action);
        await message.reply(
            await embedMsg(
                await t("commands.remindme.reminder_set", message.author, {
                    time: `<t:${Math.round(action.date.getTime() / 1000)}:R>`,
                }),
                await t("commands.remindme.title", message.author),
                "SUCCESS",
            ),
        );
    },
} as Command;
