/* eslint-disable @typescript-eslint/no-non-null-assertion */
import { Command, embed, embedMsg, resolveUser } from "lib";
import { t } from "../../modules/i18n.js";
import { API, UserBadges, UserFlags } from "revolt.js";

export default {
    name: "whois",
    aliases: ["userinfo"],
    description: "Fetch info about a user",
    async run(message, args, client) {
        const target = args[0] || message.authorId || "";
        const data = await resolveUser(
            target,
            client,
            message.channel?.serverId,
        );

        if (!data.user) {
            return await message.reply(
                embedMsg(
                    await t("commands.whois.not_found", message.author),
                    await t("commands.whois.title", message.author),
                    "ERROR",
                ),
            );
        }

        const profile = await data.user.fetchProfile().catch((e) => null);

        const flags =
            Object.entries(UserFlags)
                .filter((flag) => data.user!.flags & (flag[1] as number))
                .map((flag) => `\`${flag[0]}\``)
                .join(", ") || "`None`";
        const badges =
            Object.entries(UserBadges)
                .filter((badge) => data.user!.badges & (badge[1] as number))
                .map((badge) => `\`${badge[0]}\``)
                .join(", ") || "`None`";
        const embeds: API.SendableEmbed[] = [];

        embeds.push(
            embed(
                [
                    await t("commands.whois.data.id", message.author, {
                        id: data.user.id,
                    }),
                    await t("commands.whois.data.displayname", message.author, {
                        displayname: data.user.displayName || "*unset*",
                    }),
                    await t("commands.whois.data.username", message.author, {
                        username: data.user.username,
                    }),
                    await t(
                        "commands.whois.data.discriminator",
                        message.author,
                        {
                            discriminator: "#" + data.user.discriminator,
                        },
                    ),
                    await t("commands.whois.data.created", message.author, {
                        ts: Math.round(data.user.createdAt.getTime() / 1000),
                    }),
                    await t("commands.whois.data.flags", message.author, {
                        flags,
                        flagsNum: data.user.flags || 0,
                    }),
                    await t("commands.whois.data.badges", message.author, {
                        badges,
                        badgesNum: data.user.badges || 0,
                    }),
                    await t("commands.whois.data.icons.text", message.author, {
                        links:
                            // I know this is scuffed but the previous approach
                            // just didn't work and this trash does so it's staying
                            (
                                await Promise.all(
                                    [
                                        {
                                            key: "global",
                                            url: data.user.animatedAvatarURL,
                                        },
                                        {
                                            key: "server",
                                            url: data.member?.animatedAvatarURL,
                                        },
                                        {
                                            key: "banner",
                                            url: profile?.background
                                                ? `${client.configuration?.features.autumn.url}/${profile.background.tag}/${profile.background._id}`
                                                : null,
                                        },
                                    ]
                                        .filter((i) => i.url)
                                        .map(
                                            async (i) =>
                                                `[${await t(
                                                    `commands.whois.data.icons.${i.key}`,
                                                    message.author,
                                                )}](${i.url})`,
                                        ),
                                )
                            ).join(" | "),
                    }),
                    data.user.privileged
                        ? "\n> " +
                          (await t(
                              "commands.whois.data.privileged",
                              message.author,
                          ))
                        : null,
                ]
                    .filter((i) => i)
                    .join("\n"),
                await t("commands.whois.title", message.author),
                "INFO",
            ),
        );

        if (data.member) {
            embeds.push(
                embed(
                    [
                        data.member.nickname
                            ? await t(
                                  "commands.whois.data.nickname",
                                  message.author,
                                  {
                                      nickname: data.member.nickname,
                                  },
                              )
                            : null,
                        await t("commands.whois.data.colour", message.author, {
                            colour: data.member.roleColour || "None",
                        }),
                        await t("commands.whois.data.ranking", message.author, {
                            rank: data.member.ranking,
                            own_rank: message.member?.ranking || "Unknown",
                        }),
                        await t("commands.whois.data.roles", message.author, {
                            roles:
                                data.member.roles
                                    .map(
                                        (id) =>
                                            `\`${
                                                message.server?.roles.get(id)
                                                    ?.name ?? id
                                            }\``,
                                    )
                                    .join(", ") || "`None`",
                        }),
                        data.user.id == data.member.server?.ownerId
                            ? "#####\n> " +
                              (await t(
                                  "commands.whois.data.owner",
                                  message.author,
                              ))
                            : null,
                        message.member &&
                        message.authorId != data.user.id &&
                        message.member.ranking != data.member.ranking
                            ? "#####\n> " +
                              (data.member.inferiorTo(message.member)
                                  ? await t(
                                        "commands.whois.data.inferior",
                                        message.author,
                                    )
                                  : message.member.inferiorTo(data.member)
                                  ? await t(
                                        "commands.whois.data.superior",
                                        message.author,
                                    )
                                  : null)
                            : null,
                    ]
                        .filter((i) => i)
                        .join("\n"),
                    await t("commands.whois.title_member", message.author),
                    "INFO",
                ),
            );
        }

        if (data.user.bot?.owner) {
            embeds.push(
                embed(
                    [
                        await t(
                            "commands.whois.data.bot_owner",
                            message.author,
                            {
                                id: data.user.bot.owner,
                            },
                        ),
                    ]
                        .filter((i) => i)
                        .join("\n"),
                    await t("commands.whois.title_bot", message.author),
                    "INFO",
                ),
            );
        }

        return await message.reply({ embeds });
    },
} as Command;
