import {
    Command,
    DBInfraction,
    InfractionType,
    createInfraction,
    embed,
    generateInfractionId,
    separateUser,
} from "lib";
import { t } from "../../modules/i18n.js";
import { API } from "revolt.js";
import { checkConditions, sendInfractionLog } from "../../util/infractions.js";
import { decodeTime } from "ulid";

export default {
    name: "warn",
    aliases: [],
    description: "Add an infraction",
    permissions: ["KickMembers"],
    serverOnly: true,
    async run(message, args, client, context) {
        if (!message.authorId) throw new Error("authorId is not set");
        if (!message.server?.id) throw new Error("server.id is not set");

        const { reason, users, certain, notFound } = await separateUser(
            message,
            args,
            client,
            false,
        );

        if (
            !(await checkConditions(message, client, users, notFound, certain))
        ) {
            return;
        }

        const embeds: API.SendableEmbed[] = [];
        for (const user of users) {
            try {
                const infraction: DBInfraction = {
                    _id: {
                        infraction: await generateInfractionId(
                            message.server,
                            client.db,
                        ),
                        server: message.server.id,
                    },
                    date: new Date(decodeTime(message.id)),
                    type: InfractionType.Warn,
                    reason,
                    user: user.id,
                    moderator: message.authorId,
                    context: {
                        message: message.id,
                        channel: message.channelId,
                        server: message.channel?.serverId,
                    },
                };

                const count = await createInfraction(
                    infraction,
                    message.server,
                    client,
                );
                embeds.push(
                    embed(
                        await t("commands.warn.user_warned", message.author, {
                            id: infraction._id.infraction,
                            user: infraction.user,
                            reason: infraction.reason
                                ? `\`${infraction.reason}\``
                                : "*None*",
                            count,
                        }),
                        await t(
                            "infractions.infraction_created",
                            message.author,
                        ),
                        "SUCCESS",
                    ),
                );
                await sendInfractionLog(infraction, context.serverConfig);
            } catch (e) {
                embeds.push(
                    embed(
                        `\`\`\`js\n${e}\n\`\`\``,
                        await t("error_generic", message.author),
                    ),
                );
            }
        }

        await message.reply({
            content: await t("infractions.operation_completed", message.author),
            embeds,
        });
    },
} as Command;
