import { ServerMember } from "revolt.js";
import { client, logger } from "../index.js";
import { enabledInServer } from "lib";

const handleMemberJoin = async (member: ServerMember) => {
    logger.info(`Handling join event for ${JSON.stringify(member.id)}`);
    if (member.server && !(await enabledInServer(member.server, client))) {
        return logger.info(
            `Not enabled in server ${member.server.id}, ignoring join event`,
        );
    }
    try {
        const memberInfo = await client.db.members.findOne({
            _id: { user: member.id.user, server: member.id.server },
        });
        if (!memberInfo) return logger.debug("Did not find member info");

        // Assign existing timeout on join because Revolt forgets them for whatever reason
        if (
            member.server?.havePermission("TimeoutMembers") &&
            memberInfo.timeout &&
            memberInfo.timeout.getTime() > Date.now()
        ) {
            logger.debug("Found existing timeout for joined member");
            await member.edit({ timeout: memberInfo.timeout.toISOString() });
        }
    } catch (e) {
        logger.error("Failed to handle member join");
        console.log(e);
    }
};

client.on("serverMemberJoin", handleMemberJoin);
