import { MongoClient } from "mongodb";
import { createClient } from "redis";
import { createLogger } from "./logger.js";
import { DBServer } from "../types/database/servers.js";
import { DBUser } from "../types/database/users.js";
import { DBInfraction } from "../types/database/infraction.js";
import { DBTimedAction } from "../types/database/timed_action.js";
import { DBServerLog } from "../types/database/server_logs.js";
import { DBMember } from "../types/database/members.js";
import { DBIncrementingIDs } from "../types/database/incrementing_ids.js";
import { DBAuthToken } from "../types/database/auth_tokens.js";
import { DBUserAuditLog } from "../types/database/user_audit_log.js";

export async function connectDb(uri: string, database: string) {
    const logger = createLogger();
    try {
        logger.info("MongoDB connecing...");
        const client = new MongoClient(uri);
        await client.connect();
        logger.done("Connected to database!");

        const db = client.db(database);

        return {
            incrementingIDs:
                db.collection<DBIncrementingIDs>("incrementing_ids"),
            userAuditLog: db.collection<DBUserAuditLog>("user_audit_log"),
            timedActions: db.collection<DBTimedAction>("timed_actions"),
            infractions: db.collection<DBInfraction>("infractions"),
            serverLogs: db.collection<DBServerLog>("server_logs"),
            authTokens: db.collection<DBAuthToken>("auth_tokens"),
            servers: db.collection<DBServer>("servers"),
            members: db.collection<DBMember>("members"),
            users: db.collection<DBUser>("users"),
            mongodb: client,
        };
    } catch (e) {
        logger.error(`Failed to connect to database!`);
        throw `${e}`;
    }
}

export async function connectRedis(
    uri: string,
): Promise<ReturnType<typeof createClient>> {
    const logger = createLogger();
    try {
        logger.info("Redis connecting...");
        const redis = createClient({
            url: uri,
        });
        await redis.connect();
        logger.done("Connected to redis!");

        return redis;
    } catch (e) {
        logger.error("Failed to connect to redis!");
        throw `${e}`;
    }
}

export type AutomodDatabase = Awaited<ReturnType<typeof connectDb>>;
export type AutomodRedis = Awaited<ReturnType<typeof connectRedis>>;
