import { Message } from "revolt.js";
import { Permission } from "revolt.js/lib/esm/permissions/definitions";
import { AutomodClient } from "./AutomodClient";
import { DBServer } from "./database/servers";
import { WithId } from "mongodb";

export enum BotPermission {
    BotAdmin,
}

export type Command = {
    name: string; // Primary name of the command
    aliases?: string[]; // Optional alias names
    description: string; // Short description of what the command does
    permissions?: (keyof typeof Permission | keyof typeof BotPermission)[]; // Static permissions required to execute the command
    serverOnly?: boolean; // If true, the command can only be used in servers
    run: (
        message: Message,
        args: string[],
        client: AutomodClient,
        context: CommandContext,
    ) => Promise<any>;
};

export type CommandContext = {
    usedPrefix: string; // The prefix used to run the command, can also be <@botid>
    serverPrefix: string; // Same as `usedPrefix` but will always be the server prefix, even if bot @mention was used
    commandName: string; // The command name used to trigger the command, can be set to one of its aliases
    serverConfig: WithId<DBServer> | null; // The server configuration, if available
};
