export type DBInfraction = {
    _id: {
        // Infraction ID. Incrementing number, per server number pool
        infraction: string;

        // ID of the server the infraction belongs to
        server: string;
    };

    // Since we don't use ULIDs, we need to store the creation date separately
    date: Date;

    // ID of the user the infraction applies to
    user: string;

    // ID of the user who performed the action
    moderator: string | "System";

    // The optional reason provided by the moderator
    reason: string | null;

    // Whether this infraction has been revoked by a moderator
    revoked?: boolean;

    // IDs pointing to the message that triggered the action
    context: {
        message: string;
        channel: string;
        server?: string;
    } | null;
} & (
    | {
          type: InfractionType.BanTemporary | InfractionType.Timeout;

          // Expiry of the action
          expires: Date;
      }
    | {
          // The type of the infraction
          type: InfractionType;
      }
);

export enum InfractionType {
    Warn = "warn",
    Kick = "kick",
    Timeout = "timeout",
    BanTemporary = "ban_temporary",
    BanPermanent = "ban_permanent",
}
