import { API, TextEmbed } from "revolt.js";
import { DBInfraction } from "./infraction";

export type LoggedAttachment = {
    id: string;
    autumnTag?: string;
    filename?: string;
    spoiler: boolean;
    size?: number;
    contentType?: string;
    type: API.File["metadata"]["type"];
};

export type LoggedMessage = {
    author: {
        id?: string;
        username?: string;
        discriminator?: string;
        displayname?: string;
        bot?: string;
    };
    content?: string;
    files?: LoggedAttachment[];
    embeds?: TextEmbed[];
};

export type DBServerLog = {
    // Log ID
    _id: string;

    // Associated server
    server: string;

    // The user who has triggered this event (e.g. moderator, message author)
    user?: string;

    // Info on log delivery to text channel. Unset if log channel is not set
    delivery?: {
        state: "failed" | "success" | "no_channel";
        error?: string;
    };
} & (
    | ({
          category: ServerLogCategory.ModAction;
          infraction: DBInfraction["_id"];
      } & (
          | {
                type:
                    | ModActionType.CreateInfraction
                    | ModActionType.RemoveInfraction
                    | ModActionType.TempBanExpire;
            }
          | {
                type: ModActionType.EditInfraction;
                oldReason: string | null;
            }
      ))
    | ({
          category: ServerLogCategory.Message;
          channel: string;
      } & (
          | {
                type: MessageLogType.Edit;
                oldContent?: string;
                content: string;
                messageId: string;
            }
          | ({
                type: MessageLogType.Delete;
                messageId: string;
            } & LoggedMessage)
          | {
                type: MessageLogType.DeleteBulk;
                messages: LoggedMessage[];
            }
      ))
);

export enum ServerLogCategory {
    ModAction = "mod_action",
    Message = "message",
}

export enum ModActionType {
    CreateInfraction = "create_infraction",
    RemoveInfraction = "remove_infraction",
    EditInfraction = "edit_infraction",
    TempBanExpire = "temp_ban_expire",
}

export enum MessageLogType {
    Edit = "edit",
    Delete = "delete",
    DeleteBulk = "bulk_delete",
}
