import { DBInfraction } from "./infraction";

/**
 * An action that should be executed at a given time
 */
export type DBTimedAction = {
    // ULID of the action
    _id: string;

    // When this action will be executed
    date: Date;

    state: TimedActionState;

    // If state is `Failed`, we can store the error message here
    error?: string;
} & (
    | {
          action: TimedActionType.Unban;
          infraction: DBInfraction["_id"];
      }
    | {
          action: TimedActionType.Reminder;

          // The user that requested a reminder message
          user: string;

          // The text the user wants to be reminded of
          reminder: string;

          // Message link that points to the message where the reminder was created
          message: string;
      }
);

export enum TimedActionType {
    Unban = "Unban",
    Reminder = "Reminder",
}

export enum TimedActionState {
    // The action has not yet been executed
    Pending = "Pending",

    // The action executed successfully
    Success = "Success",

    // The action failed to execute
    Failed = "Failed",

    // The action was cancelled
    Cancelled = "Cancelled",
}
